/**
 * Module の読み込み
 */
var express = require('express')
  , routes = require('./routes')
  , WebSocketServer = require('websocket').server

var app = module.exports = express.createServer();

// Express の設定部. viewのディレクトリパスなどを設定

app.configure(function(){
  app.set('views', __dirname + '/views');
  app.set('view engine', 'ejs');
  app.use(express.bodyParser());
  app.use(express.methodOverride());
  app.use(app.router);
  app.use(express.static(__dirname + '/public'));
});


// developmentモードの時の設定
app.configure('development', function(){
  app.use(express.errorHandler({ dumpExceptions: true, showStack: true }));
});

// productionモードの時の設定
app.configure('production', function(){
  app.use(express.errorHandler());
});

// ルーティング設定. routes/index.js内のメソッドが実行される

app.get('/', routes.index);


// WebSocket パート。インスタンスを生成する
// -------------------
var wsServer = new WebSocketServer({
  httpServer: app
  , fragmentOutgoingMessage: false // for avoiding a FF7 alpha's bug
});

// クライアントコネクションをストアする配列
var conns = [];

// クライアントからのコネクション要求に応じて、発動される
// イベントハンドラ
wsServer.on('request', function(req) {
  // コネクション要求をアクセプトする。Originチェックを行う場合は
  // req.originをチェックするコーディングが必要
  var conn = req.accept('', req.origin);

  // conns配列に、クライアントコネクションの情報を格納
  conns.push(conn);

  // 接続情報をログに出力
  console.log(conn.remoteAddress + " connected - Protocol Version " + conn.websocketVersion);


  // 受信したメッセージを、他の接続ユーザーにブロードキャストする
  conn.on('message', function(message) {
    if (message.type === 'utf8') {
      for (var i = 0, l = conns.length; i < l; i += 1) {
        if( conns[i] !== conn ) { // 自分自身はブロードキャストから除く
          conns[i].sendUTF(message.utf8Data);
        }
      }
    }
  });


  // コネクション切断時のハンドラ
  conn.on('close', function(){
    // ログに出力
    console.log(conn.remoteAddress + " disconnected");

    // conns配列から該当のコネクション情報を削除する
    var index = conns.indexOf(conn);
    if (index  !== -1 ) {
      conns.splice(index, 1);
    }
  });
});

// listen開始
app.listen(3000);
console.log("Express server listening on port %d in %s mode", app.address().port, app.settings.env);
